@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2006 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2006 Zope Foundation and Contributors.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -12,7 +12,7 @@
 ##############################################################################
 """Formlib form base classes.
 
-$Id: form.py 107334 2009-12-30 01:02:45Z hannosch $
+$Id: form.py 113211 2010-06-06 15:26:29Z hannosch $
 """
 
 from datetime import datetime
@@ -21,16 +21,10 @@ from sets import Set
 from AccessControl.SecurityInfo import ClassSecurityInfo
 from App.class_init import InitializeClass
 from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
-# BBB Zope 2.12
-try:
-    from five.formlib.formbase import PageAddForm
-    from five.formlib.formbase import PageDisplayForm
-    from five.formlib.formbase import PageForm
-except ImportError:
-    from Products.Five.formlib.formbase import PageAddForm
-    from Products.Five.formlib.formbase import PageDisplayForm
-    from Products.Five.formlib.formbase import PageForm
 
+from five.formlib.formbase import PageAddForm
+from five.formlib.formbase import PageDisplayForm
+from five.formlib.formbase import PageForm
 from zope.component import adapts
 from zope.component import getUtility
 from zope.component.interfaces import IFactory
