@@ -26,15 +26,22 @@ from StringIO import StringIO
 
 from DocumentTemplate.DT_Util import html_quote
 from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
+from zope.component import getSiteManager
+from zope.interface import implements
+from zope.interface.verify import verifyClass
 
+from Products.CMFCore.interfaces import ILinebreakNormalizer
 from Products.CMFCore.testing import ConformsToContent
 from Products.CMFCore.tests.base.content import BASIC_HTML
+from Products.CMFCore.tests.base.content import BASIC_ReST
 from Products.CMFCore.tests.base.content import BASIC_STRUCTUREDTEXT
 from Products.CMFCore.tests.base.content import DOCTYPE
 from Products.CMFCore.tests.base.content import ENTITY_IN_TITLE
 from Products.CMFCore.tests.base.content import FAUX_HTML_LEADING_TEXT
 from Products.CMFCore.tests.base.content import HTML_TEMPLATE
-from Products.CMFCore.tests.base.content import SIMPLE_HTML
+from Products.CMFCore.tests.base.content import ReST_NO_HEADERS
+from Products.CMFCore.tests.base.content import ReST_NO_HEADERS_BUT_COLON
+from Products.CMFCore.tests.base.content import ReST_WITH_HTML
 from Products.CMFCore.tests.base.content import SIMPLE_STRUCTUREDTEXT
 from Products.CMFCore.tests.base.content import SIMPLE_XHTML
 from Products.CMFCore.tests.base.content import STX_NO_HEADERS
@@ -42,7 +49,7 @@ from Products.CMFCore.tests.base.content import STX_NO_HEADERS_BUT_COLON
 from Products.CMFCore.tests.base.content import STX_WITH_HTML
 from Products.CMFCore.tests.base.dummy import DummySite
 from Products.CMFCore.tests.base.testcase import RequestTest
-from Products.CMFCore.tests.base.tidata import FTIDATA_CMF15
+from Products.CMFCore.tests.base.tidata import FTIDATA_CMF
 from Products.CMFCore.TypesTool import FactoryTypeInformation as FTI
 from Products.CMFCore.TypesTool import TypesTool
 from Products.CMFDefault import utils
@@ -59,18 +66,24 @@ class RequestTestBase(RequestTest):
         return self._getTargetClass()(*args, **kw)
 
 
+class DummyLinebreakNormalizer(object):
+
+    implements(ILinebreakNormalizer)
+
+    def __init__(self, before, after):
+        self.before = before
+        self.after = after
+
+    def normalizeIncoming(self, obj, text):
+        return text.replace(self.before, self.after)
+
+    def normalizeOutgoing(self, ob, text):
+        return text.replace(self.before, self.after)
+
+
 class DocumentTests(ConformsToContent, RequestTestBase):
 
-    def test_z2interfaces(self):
-        from Interface.Verify import verifyClass
-        from Products.CMFDefault.interfaces.Document import IDocument
-        from Products.CMFDefault.interfaces.Document import IMutableDocument
-
-        verifyClass(IDocument, self._getTargetClass())
-        verifyClass(IMutableDocument, self._getTargetClass())
-
-    def test_z3interfaces(self):
-        from zope.interface.verify import verifyClass
+    def test_interfaces(self):
         from Products.CMFDefault.interfaces import IDocument
         from Products.CMFDefault.interfaces import IMutableDocument
 
@@ -267,6 +280,75 @@ class DocumentTests(ConformsToContent, RequestTestBase):
         self.failIf( d.CookedBody().find('<h1') >= 0 )
         self.failUnless( d.CookedBody().find('<h2') >= 0 )
 
+    def test_ReStructuredText(self):
+        self.REQUEST['BODY'] = BASIC_ReST
+        d = self._makeOne('foo')
+        d.PUT(self.REQUEST, self.RESPONSE)
+        d.edit(text_format='restructured-text', text=BASIC_ReST)
+
+        self.failUnless( hasattr(d, 'cooked_text') )
+        self.assertEqual( d.Format(), 'text/plain' )
+        self.assertEqual( d.Title(), 'My Document' )
+        self.assertEqual( d.Description(), 'A document by me' )
+        self.assertEqual( len(d.Contributors()), 3 )
+
+        # CookedBody() for consistency
+        self.failUnless( d.CookedBody().find('<p>') >= 0 )
+
+        #  override default
+        self.failUnless( d.CookedBody(rest_level=0).find('<h1') >= 0 )
+
+        # we should check for the list
+        self.failUnless( d.CookedBody().find('<ul') >= 0 )
+
+        # Make sure extra HTML is NOT found
+        self.failUnless( d.cooked_text.find('<title>') < 0 )
+        self.failUnless( d.cooked_text.find('<body>') < 0 )
+
+        # test subject/keyword headers
+        subj = list(d.Subject())
+        self.assertEqual( len(subj), 4 )
+        subj.sort()
+        self.assertEqual( subj, [ 'content management'
+                                , 'framework'
+                                , 'unit tests'
+                                , 'zope'
+                                ] )
+
+    def test_EditReStructuredTextWithHTML(self):
+        d = self._makeOne('foo')
+        d.edit(text_format='restructured-text', text=ReST_WITH_HTML)
+
+        self.assertEqual(d.Format(), 'text/plain')
+        self.assertEqual(d.get_size(), len(ReST_WITH_HTML))
+
+    def test_ReST_Levels(self):
+        d = self._makeOne('foo')
+        d.edit(text_format='restructured-text', text=BASIC_ReST)
+        d.CookedBody(rest_level=0, setlevel=True)
+        self.assertEqual( d._rest_level, 0)
+
+        ct = d.CookedBody()
+
+        # ReST always augments the level by one
+        self.failUnless( d.CookedBody().find('<h1') >= 0 )
+        self.assertEqual( d._rest_level, 0)
+
+        ct = d.CookedBody(rest_level=1)
+        self.failIf( ct.find('<h1') >= 0 )
+        self.failUnless( ct.find('<h2') >= 0 )
+        self.assertEqual( d._rest_level, 0 )
+
+        ct = d.CookedBody(rest_level=1, setlevel=1)
+        self.failIf( ct.find('<h1') >= 0 )
+        self.failUnless( ct.find('<h2') >= 0 )
+        self.assertEqual( d._rest_level, 1 )
+
+        ct = d.CookedBody()
+        self.assertEqual( d._rest_level, 1 )
+        self.failIf( d.CookedBody().find('<h1') >= 0 )
+        self.failUnless( d.CookedBody().find('<h2') >= 0 )
+
     def test_Init(self):
         self.REQUEST['BODY']=BASIC_STRUCTUREDTEXT
         d = self._makeOne('foo')
@@ -321,6 +403,29 @@ class DocumentTests(ConformsToContent, RequestTestBase):
         self.failUnless( 'plain' in d.Subject() )
         self.failUnless( 'STX' in d.Subject() )
 
+    def test_ReST_NoHeaders( self ):
+        self.REQUEST['BODY'] = ReST_NO_HEADERS
+        d = self._makeOne('foo')
+        d.editMetadata( title="Plain ReST"
+                       , description="Look, Ma, no headers!"
+                       , subject=( "plain", "ReST" )
+                       )
+        self.assertEqual( d.Format(), 'text/html' )
+        self.assertEqual( d.Title(), 'Plain ReST' )
+        self.assertEqual( d.Description(), 'Look, Ma, no headers!' )
+        self.assertEqual( len( d.Subject() ), 2 )
+        self.failUnless( 'plain' in d.Subject() )
+        self.failUnless( 'ReST' in d.Subject() )
+
+        d.PUT(self.REQUEST, self.RESPONSE)
+
+        self.assertEqual( d.Format(), 'text/plain' )
+        self.assertEqual( d.Title(), 'Plain ReST' )
+        self.assertEqual( d.Description(), 'Look, Ma, no headers!' )
+        self.assertEqual( len( d.Subject() ), 2 )
+        self.failUnless( 'plain' in d.Subject() )
+        self.failUnless( 'ReST' in d.Subject() )
+
     def test_STX_NoHeaders_but_colon( self ):
         d = self._makeOne('foo')
         d.editMetadata( title="Plain STX"
@@ -331,6 +436,16 @@ class DocumentTests(ConformsToContent, RequestTestBase):
         d.edit(text_format='structured-text', text=STX_NO_HEADERS_BUT_COLON)
         self.assertEqual( d.EditableBody(), STX_NO_HEADERS_BUT_COLON )
 
+    def test_ReST_NoHeaders_but_colon( self ):
+        d = self._makeOne('foo')
+        d.editMetadata( title="Plain ReST"
+                       , description="Look, Ma, no headers!"
+                       , subject=( "plain", "ST" )
+                       )
+
+        d.edit(text_format='restructured-text', text=ReST_NO_HEADERS_BUT_COLON)
+        self.assertEqual( d.EditableBody(), ReST_NO_HEADERS_BUT_COLON )
+
     def test_ZMI_edit( self ):
         d = self._makeOne('foo')
         d.editMetadata( title="Plain STX"
@@ -356,6 +471,12 @@ class DocumentTests(ConformsToContent, RequestTestBase):
         d.setFormat( d.Format() )
         self.assertEqual( d.text_format, 'structured-text' )
 
+        d.setFormat('restructured-text')
+        self.assertEqual( d.text_format, 'restructured-text' )
+        self.assertEqual( d.Format(), 'text/plain' )
+        d.setFormat( d.Format() )
+        self.assertEqual( d.text_format, 'restructured-text' )
+
         d.setFormat('html')
         self.assertEqual( d.text_format, 'html' )
         self.assertEqual( d.Format(), 'text/html' )
@@ -371,6 +492,67 @@ class DocumentTests(ConformsToContent, RequestTestBase):
         self.assertEqual( d.Format(), 'text/plain' )
         self.assertEqual( d.text_format, 'structured-text' )
 
+    def test_normalize_linebreaks_incoming(self):
+        # New feature: Line breaks can be normalized on the way in
+        # and on the way out, *if* a specific utility exists.
+        # (http://www.zope.org/Collectors/CMF/174)
+        d = self._makeOne('foo', text='')
+        LF_TEXT = 'This\nis\nsome\ntext'
+        CRLF_TEXT = 'This\r\nis\r\nsome\r\ntext'
+
+        # First case, no normalizer installed. Text will stay the same.
+        d._edit(LF_TEXT)
+        self.assertEquals(d.text, LF_TEXT)
+
+        d._edit(CRLF_TEXT)
+        self.assertEquals(d.text, CRLF_TEXT)
+        
+        # Now register a normalizer that always replaces CRLF with LF
+        # When I pass in CRLFs, the resulting text will only have LFs.
+        crlf_to_lf = DummyLinebreakNormalizer('\r\n', '\n')
+        sm = getSiteManager()
+        sm.registerUtility(crlf_to_lf, ILinebreakNormalizer)
+
+        d._edit(CRLF_TEXT)
+        self.assertEquals(d.text, LF_TEXT)
+
+        d._edit(LF_TEXT)
+        self.assertEquals(d.text, LF_TEXT)
+
+        # cleanup
+        sm.unregisterUtility(crlf_to_lf, ILinebreakNormalizer)
+
+    def test_normalize_linebreaks_outgoing(self):
+        # New feature: Line breaks can be normalized on the way in
+        # and on the way out, *if* a specific utility exists.
+        # (http://www.zope.org/Collectors/CMF/174)
+        d = self._makeOne('foo', text='')
+        LF_TEXT = 'This\nis\nsome\ntext'
+        CRLF_TEXT = 'This\r\nis\r\nsome\r\ntext'
+        HEADERS = """Title: \r\nSubject: \r\nPublisher: No publisher\r\nDescription: \r\nContributors: \r\nEffective_date: None\r\nExpiration_date: None\r\nType: Unknown\r\nFormat: text/plain\r\nLanguage: \r\nRights: \r\nSafetyBelt: \r\n\r\n"""
+
+        # First case, no normalizer installed. Text will stay the same.
+        d.text = LF_TEXT
+        self.assertEquals(d.manage_FTPget(), HEADERS + LF_TEXT)
+
+        d.text = CRLF_TEXT
+        self.assertEquals(d.manage_FTPget(), HEADERS + CRLF_TEXT)
+
+        # Now register a normalizer that always replaces CRLF with LF
+        # When I pass in CRLFs, the resulting text will only have LFs.
+        crlf_to_lf = DummyLinebreakNormalizer('\r\n', '\n')
+        sm = getSiteManager()
+        sm.registerUtility(crlf_to_lf, ILinebreakNormalizer)
+
+        d.text = (CRLF_TEXT)
+        self.assertEquals(d.manage_FTPget(), HEADERS + LF_TEXT)
+
+        d.text = LF_TEXT
+        self.assertEquals(d.manage_FTPget(), HEADERS + LF_TEXT)
+
+        # cleanup
+        sm.unregisterUtility(crlf_to_lf, ILinebreakNormalizer)
+
 
 class DocumentFTPGetTests(RequestTestBase):
 
@@ -382,7 +564,7 @@ class DocumentFTPGetTests(RequestTestBase):
         self.REQUEST['BODY']=BASIC_HTML
 
         ttool = self.site._setObject( 'portal_types', TypesTool() )
-        fti = FTIDATA_CMF15[0].copy()
+        fti = FTIDATA_CMF[0].copy()
         del fti['id']
         ttool._setObject( 'Document', FTI('Document', **fti) )
 
@@ -526,6 +708,22 @@ class DocumentPUTTests(RequestTestBase):
         self.assertEqual( d.Format(), 'text/plain' )
         self.assertEqual( r.status, 204 )
 
+    def test_PutReStructuredTextWithHTML(self):
+        self.REQUEST['BODY'] = ReST_WITH_HTML
+        d = self._makeOne('foo')
+        r = d.PUT(self.REQUEST, self.RESPONSE)
+
+        self.assertEqual( d.Format(), 'text/plain' )
+        self.assertEqual( r.status, 204 )
+
+    def test_PutReStructuredText(self):
+        self.REQUEST['BODY'] = BASIC_ReST
+        d = self._makeOne('foo')
+        r = d.PUT(self.REQUEST, self.RESPONSE)
+
+        self.assertEqual( d.Format(), 'text/plain' )
+        self.assertEqual( r.status, 204 )
+
     def test_PutHtmlWithDoctype(self):
         html = '%s\n\n  \n   %s' % (DOCTYPE, BASIC_HTML)
         self.REQUEST['BODY'] = html
